
/* Chrysalide - Outil d'analyse de fichiers binaires
 * timestamp.h - prototypes pour l'encodage par plage unique d'une chaîne de caractères
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_DB_MISC_TIMESTAMP_H
#define _ANALYSIS_DB_MISC_TIMESTAMP_H


#include <stdbool.h>
#include <stdint.h>


#include "../../../common/sqlite.h"



/* Représentation d'un horodatage */
typedef uint64_t timestamp_t;


/* Valeurs particulières */
#define TIMESTAMP_ALL_ACTIVE 0
#define TIMESTAMP_ALL_INACTIVE 1


/* Obtient un horodatage initialisé au moment même. */
void init_timestamp(timestamp_t *);

/* Définit si un horodatage est plus récent qu'un autre ou non. */
bool timestamp_is_younger(timestamp_t, timestamp_t);

/* Effectue la comparaison entre deux horodatages. */
int cmp_timestamp(const timestamp_t *, const timestamp_t *);

/* Importe la définition d'un horodatage. */
bool recv_timestamp(timestamp_t *, int, int);

/* Exporte la définition d'un horodatage. */
bool send_timestamp(const timestamp_t *, int, int);



/* --------------------- MANIPULATIONS AVEC UNE BASE DE DONNEES --------------------- */


/* Définition du tronc commun pour les créations SQLite */
#define SQLITE_TIMESTAMP_CREATE(n)      \
    n " INTEGER"

/* Constitue les champs destinés à une insertion / modification. */
bool prepare_db_statement_for_timestamp(const timestamp_t *, const char *, bound_value **, size_t *);

/* Décrit les colonnes utiles à un horodatage. */
bool setup_load_of_timestamp(const timestamp_t *, const char *, bound_value **, size_t *);

/* Charge les valeurs utiles pour un horodatage. */
bool load_timestamp(timestamp_t *, const char *, const bound_value *, size_t);



#endif  /* _ANALYSIS_DB_MISC_TIMESTAMP_H */
