
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ##FILE## - traduction d'instructions ARMv7
 *
 * Copyright (C) 2014 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


@title BL, BLX (immediate)

@encoding(T1) {

    @word 1 1 1 1 0 S(1) imm10(10) 1 1 J1(1) 1 J2(1) imm11(11)

    @syntax <imm32>

    @conv {

        I1 = NOT(J1 EOR S)
        I2 = NOT(J2 EOR S)
        imm32 = SignExtend(S:I1:I2:imm10:imm11:'0', S, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_bl_from_thumb
        link = handle_call_as_link
        post = post_process_branch_and_link_instructions

    }

    @rules {

        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(T2) {

    @word 1 1 1 1 0 S(1) imm10H(10) 1 1 J1(1) 0 J2(1) imm10L(10) H(1)

    @syntax "blx" <imm32>

    @conv {

        I1 = NOT(J1 EOR S)
        I2 = NOT(J2 EOR S)
        imm32 = SignExtend(S:I1:I2:imm10H:imm10L:'00', S, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_blx_from_thumb
        link = handle_call_as_link
        post = post_process_branch_and_link_instructions

    }

    @rules {

        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(A1) {

    @word cond(4) 1 0 1 1 imm24(24)

    @syntax <imm32>

    @conv {

        imm32 = SignExtend(imm24:'00', imm24 & 0x800000, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_bl_from_arm
        link = handle_call_as_link
        post = post_process_branch_and_link_instructions

    }

    @rules {

        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}

@encoding(A2) {

    @word 1 1 1 1 1 0 1 H(1) imm24(24)

    @syntax "blx" <imm32>

    @conv {

        imm32 = SignExtend(imm24:H:'0', imm24 & 0x800000, 32)

    }

    @hooks {

        fetch = help_fetching_with_instruction_blx_from_arm
        link = handle_call_as_link
        post = post_process_branch_and_link_instructions

    }

    @rules {

        //chk_call SetInsFlag(AIF_RETURN_POINT)

    }

}
